// iftest.c - P60xBO GPIO driver interface test suite
//
// Copyright (c) 2007 General Micro Systems, Inc.  All Rights Reserved
//
// You are hereby granted a copyright license to use, modify and
// distribute this SOFTWARE so long as the entire notice is retained
// without alteration in any modified and/or redistributed versions,
// and that such modified versions are clearly identified as such.
// No licenses are granted by implication, estopple or otherwise under
// any patents or trademarks of General Micro Systems, Inc (GMS).
//
// The SOFTWARE is provided on an "AS IS" basis and without warrenty,
// to the maximum extent permitted by applicable law.
//
// GMS DISCLAIMS ALL WARRANTIES WHETHER EXPRESS OR IMPLIED, INCLUDING
// WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE
// AND ANY WARRANTY AGAINST INFRINGEMENT WITH REGARD TO THE SOFTWARE
// (INCLUDING ANY MODIFIED VERSIONS THEREOF) AND ANY ACCOMPANYING
// WRITTEN MATERIAL.
//
// To the maximum extent permitted by applicable law, IN NO EVENT SHALL
// GMS BE LIABLE FOR ANY DAMAGE WHATSOEVER (INCLUDING WITHOUT LIMITATION,
// DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, LOSS OF
// BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM THE USE
// OR INABILITY TO USE THE SOFTWARE.  GMS assumes no resposibility for
// for the maintenance or support of the SOFTWARE
//

// Maintenance Log
//---------------------------------------------------------------------
// v1.2     02-Nov-2013  vp      improved GET functions
// v1.1     07/02/2008   ew      Ported to Windows
// v1.0     12/17/2007   ew/dp   Written
// --------------------------------------------------------------------


// include files

// include our API include file
#include "p60xbo_xgpio_if.h"


enum OPERATIONS {
    DONOTHING   = 0,
    DOINPUT     = 1,
    DOOUTPUT    = 2,
    DOHIGH      = 3,
    DOLOW       = 4,
    DORESET     = 5,
    READLEVEL   = 6,
    READDIR     = 7,
    READLREG    = 8,
    READDREG    = 9,
};


//*****************************************************************************
// Function: usage_exit
//
// Description: This function will display the usage of this program to
//					the stderr device and perform an error exit.
//
// Arguments: 		prgname - pointer to our program name
// Returns:			Does not return
// Side Effects:	N/A
//*****************************************************************************

void usage_exit(const char *prgname)
{
    fprintf(stderr,"P60xBO GPIO Interface Driver Test Suite  (V1.1)  07/02/2008\n"
          "Copyright (c) 2008 General Micro Systems, Inc.\n"
		  "All Rights Reserved\n"
		  "\nUsage: %s <options>\n"
          "\t-i\t\tset pin for input \n"
          "\t-o\t\tset pin for output \n"
          "\t-h\t\tset pin high \n"
          "\t-l\t\tset pin low \n"
          "\t-L\t\tGPIO level (0=low;1=high)\n"
          "\t-0\t\tGPIO level register value\n"
          "\t-D\t\tGPIO direction (0=out;1=in)\n"
          "\t-1\t\tGPIO direction register value\n"
          "\t-R\t\tReset Interface (0,0)\n"
          "\t-H\t\thelp message\n",
          prgname);
     exit(1);
}



int	opterr = 1;
int	optind = 1;
int	optopt;
char	*optarg;

int
getopt(argc, argv, opts)
int	argc;
char	**argv, *opts;
{
	static int sp = 1;
	register int c;
	register char *cp;

	if(sp == 1)
		if(optind >= argc ||
		   argv[optind][0] != '-' || argv[optind][1] == '\0')
			return(EOF);
        else if(strcmp(argv[optind], "--") == 0) {
			optind++;
			return(EOF);
		}
	optopt = c = argv[optind][sp];
    if(c == ':' || (cp=strchr(opts, c)) == NULL) {
        fprintf(stderr,": illegal option -- ", c);
		if(argv[optind][++sp] == '\0') {
			optind++;
			sp = 1;
		}
		return('?');
	}
	if(*++cp == ':') {
		if(argv[optind][sp+1] != '\0')
			optarg = &argv[optind++][sp+1];
		else if(++optind >= argc) {
            fprintf(stderr,": option requires an argument -- ", c);
			sp = 1;
			return('?');
		} else
			optarg = argv[optind++];
		sp = 1;
	} else {
		if(argv[optind][++sp] == '\0') {
			sp = 1;
			optind++;
		}
		optarg = NULL;
	}
	return(c);
}


/*
 *
 *  nothing elegant, but it still works
 *
 */

VOID __cdecl
main(
    __in ULONG argc,
    __in_ecount(argc) PCHAR argv[]
    )
{
    enum OPERATIONS cmd = READLEVEL;    // default to reading GPIO level
    int Do_Write = 0;                   // default to read ops only
    ULONG temp; 
    int pin_number=0;
    char *stmp;


    if (argc < 2)
  		usage_exit(argv[0]);

    for (;;) {
      int c;
      c = getopt(argc, argv, "d:riohlvDLR01H");
      switch(c) {
        case -1: break;
        case 'r': cmd      = READLEVEL;  break;
        case 'i': cmd      = DOINPUT;    break;
        case 'o': cmd      = DOOUTPUT;   break;
        case 'h': cmd      = DOHIGH;     break;
        case 'l': cmd      = DOLOW;      break;
        case 'D': cmd      = READDIR;    break;
        case 'L': cmd      = READLEVEL;  break;
        case '1': cmd      = READDREG;   break;
        case '0': cmd      = READLREG;   break;
        case 'R': cmd      = DORESET;  	 break;
        case 'H':
        default:
           usage_exit(argv[0]);
        }
        if (c== -1) break;
    }


    // First, initialize the device driver
    if (InitXGpioInterface())
            exit(1);


    // make sure we have the proper number of arguments
    // argv[2] should be the pin_number
    if (argc == 3) {
    	pin_number = strtoul(argv[2], &stmp, 0);
    	if (argv[2] == stmp) {
         perror("bad arguments");
         exit(1);
	}
    } else {
	fprintf(stderr,"No arguments, pin_number assumed '0'.\n");
    }


    // perform any signal switches here
    switch (cmd) {
        case DORESET:                   // reset outputs to zero
            ResetXGpioInterface();
            cmd      = READDIR;
            break;
        case DOINPUT:                   // set a pin to input
            if (SetPinDirection(pin_number, INPUT_DIR))
            {
                perror("SetPinDirection Input");
                exit(1);
            }
            cmd      = READDIR;
            break;
        case DOOUTPUT:                  // set a pin to output
            if (SetPinDirection(pin_number, OUTPUT_DIR))
            {
                perror("SetPinDirection Output");
                exit(1);
            }
            cmd      = READDIR;
            break;
        case DOHIGH:                    // set a pin high
            if (SetPinHigh(pin_number))
            {
                perror("SetPinHigh failed");
                exit(1);
            }
            cmd      = READLEVEL;
            break;
        case DOLOW:                     // set a pin low
            if (SetPinLow(pin_number))
            {
                perror("SetPinLow failed");
                exit(1);
            }
            cmd      = READLEVEL;
            break;
        case READLEVEL:                 // both here to avoid fault
        case READDIR:
        case READDREG:                  // both here to avoid fault
        case READLREG:
		// Donothing
	    break;
    default:
            fprintf(stderr,"No Ioctls to do. \n");
            Do_Write = 2;
            break;
    }


    // perform readback operations here for confirmation
    switch (cmd) {
        case READLEVEL:                 // read pin state
            if (GetPinState(pin_number, &temp))
                exit(1);
            fprintf(stderr,"GetPinState[%d]= %d. \n", pin_number, temp);
            break;

        case READDIR:                   // read direction state
            if (GetPinDirection(pin_number, &temp))
                exit(1);
            fprintf(stderr,"GetPinDirection[%d]= %d. \n", pin_number, temp);
            break;

        case READDREG:                 // read pin state
            if (GetRegState(1, &temp))
                exit(1);
            fprintf(stderr,"GetRegState[%d]= %02x. \n", 1, temp);
            break;

        case READLREG:                   // read direction state
            if (GetRegState(0, &temp))
                exit(1);
            fprintf(stderr,"GetRegState[%d]= %02x. \n", 0, temp);
            break;

    default:
            fprintf(stderr,"Nothing to read. \n");
            Do_Write = 2;
            break;
    }


    // close out the driver interface
    ExitXGpioInterface();

    // nice exit
    exit(0);
}
