// p60xb0_xgpio_if.h - P60xBO GPIO driver interface file
//
// Copyright (c) 2007 General Micro Systems, Inc.  All Rights Reserved
//
// You are hereby granted a copyright license to use, modify and
// distribute this SOFTWARE so long as the entire notice is retained
// without alteration in any modified and/or redistributed versions,
// and that such modified versions are clearly identified as such.
// No licenses are granted by implication, estopple or otherwise under
// any patents or trademarks of General Micro Systems, Inc (GMS).
//
// The SOFTWARE is provided on an "AS IS" basis and without warrenty,
// to the maximum extent permitted by applicable law.
//
// GMS DISCLAIMS ALL WARRANTIES WHETHER EXPRESS OR IMPLIED, INCLUDING
// WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE
// AND ANY WARRANTY AGAINST INFRINGEMENT WITH REGARD TO THE SOFTWARE
// (INCLUDING ANY MODIFIED VERSIONS THEREOF) AND ANY ACCOMPANYING
// WRITTEN MATERIAL.
//
// To the maximum extent permitted by applicable law, IN NO EVENT SHALL
// GMS BE LIABLE FOR ANY DAMAGE WHATSOEVER (INCLUDING WITHOUT LIMITATION,
// DAMAGES FOR LOSS OF BUSINESS PROFITS, BUSINESS INTERRUPTION, LOSS OF
// BUSINESS INFORMATION, OR OTHER PECUNIARY LOSS) ARISING FROM THE USE
// OR INABILITY TO USE THE SOFTWARE.  GMS assumes no resposibility for
// for the maintenance or support of the SOFTWARE
//

// Maintenance Log
//---------------------------------------------------------------------
// v0.91     12/17/2007  ew      Written
// --------------------------------------------------------------------

#ifndef __HR_XGPIO_IF_H
#define __HR_XGPIO_IF_H

// one cheap glue interface //

#include <windows.h>
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <winioctl.h>
#include "gpioctl.h"        // This defines the IOCTL constants.
#include <dontuse.h>

typedef enum _DirectionEnum  {
    OUTPUT_DIR       = 0,
    INPUT_DIR        = 1
} DirectionEnum;

typedef enum _RegisterEnum  {
    GPIO_REG       = 0,
    DIR_REG        = 1
} DirectionEnum;


static HANDLE fd_p60xbo_xgp;        // Handle to device, obtain from CreateFile

// Initializes the P60xBO XGPIO interface to the kernel driver
static int InitXGpioInterface(void)
   {
    fd_p60xbo_xgp = CreateFile(
                    "\\\\.\\XGpioDev",           // Open the Device "file"
					GENERIC_READ | GENERIC_WRITE,
					0,
                    NULL,
                    OPEN_EXISTING,
                    0,
                    NULL);

    if (fd_p60xbo_xgp == INVALID_HANDLE_VALUE)        // Was the device opened?
    {
        printf("Unable to open the device.\n");
        exit(1);
    }

    return 0;
   }

static int SetPinDirection(int pin_number, DirectionEnum direction)  // Set the direction of the GPIO pin
                                                            //  and return a success/error indicator
   {
        XGPIOPORT_WRITE_INPUT tbuf;
        ULONG                 ReturnedLength; // Number of bytes returned in output buffer
        BOOL                  IoctlResult;

        if ( (pin_number <0) || (pin_number >7)) {
            printf("ERROR:  wrong pin number %d (0..7)\n", pin_number);
            return(-1);
        }

        if ( (direction != INPUT_DIR)  && (direction != OUTPUT_DIR)) {
            return(-1);
        }

        tbuf.CharData[0] = direction << pin_number;
        tbuf.CharData[1] = 1 << pin_number;

//       if (ioctl(fd_p60xbo_xgp, P60xBO_XGPIO_SET_DIR, &tbuf))
        if (!DeviceIoControl(
                        fd_p60xbo_xgp,                 // Handle to device
                        IOCTL_P60xBO_XGPIO_SET_DIR,    // IO Control code for Write
                        &tbuf,                         // Buffer to driver.  Holds port & data.
                        sizeof(XGPIOPORT_WRITE_INPUT), // Length of buffer in bytes.
                        NULL,                          // Buffer from driver.   Not used.
                        0,                             // Length of buffer in bytes.
                        &ReturnedLength,               // Bytes placed in outbuf.  Should be 0.
                        NULL                           // NULL means wait till I/O completes.
                        ))                            // Did the IOCTL succeed?
        {
//                Ioctl failed
                        return(-1);
        }

        return 0;
   }



static int SetPinHigh(int pin_number) //Given a GPIO pin, set it high and return a success/error indicator
   {
        XGPIOPORT_WRITE_INPUT tbuf;
        ULONG                 ReturnedLength; // Number of bytes returned in output buffer
        BOOL                  IoctlResult;

        if ( (pin_number <0) || (pin_number >7)) {
            printf("ERROR:  wrong pin number %d (0..7)\n", pin_number);
            return(-1);
        }

        tbuf.CharData[0] = 1 << pin_number;
        tbuf.CharData[1] = 1 << pin_number;

//        if (ioctl(fd_p60xbo_xgp, P60xBO_XGPIO_SET_LEVEL, &tbuf))
        if (!DeviceIoControl(
                        fd_p60xbo_xgp,                 // Handle to device
                        IOCTL_P60xBO_XGPIO_SET_LEVEL,  // IO Control code for Write
                        &tbuf,                         // Buffer to driver.  Holds port & data.
                        sizeof(XGPIOPORT_WRITE_INPUT), // Length of buffer in bytes.
                        NULL,                          // Buffer from driver.   Not used.
                        0,                             // Length of buffer in bytes.
                        &ReturnedLength,               // Bytes placed in outbuf.  Should be 0.
                        NULL                           // NULL means wait till I/O completes.
                        ))                            // Did the IOCTL succeed?
        {
//                Ioctl failed
                        return(-1);
        }

        return 0;
   }




static int SetPinLow(int pin_number) //Given a GPIO pin, set it low and return a success/error indicator
   {
        XGPIOPORT_WRITE_INPUT tbuf;
        ULONG                 ReturnedLength; // Number of bytes returned in output buffer
        BOOL                  IoctlResult;

        if ( (pin_number <0) || (pin_number >7)) {
            printf("ERROR:  wrong pin number %d (0..7)\n", pin_number);
            return(1);
        }

        tbuf.CharData[0] = 0;
        tbuf.CharData[1] = 1 << pin_number;

//        if (ioctl(fd_p60xbo_xgp, P60xBO_XGPIO_SET_LEVEL, &tbuf))
        if (!DeviceIoControl(
                        fd_p60xbo_xgp,                 // Handle to device
                        IOCTL_P60xBO_XGPIO_SET_LEVEL,  // IO Control code for Write
                        &tbuf,                         // Buffer to driver.  Holds port & data.
                        sizeof(XGPIOPORT_WRITE_INPUT), // Length of buffer in bytes.
                        NULL,                          // Buffer from driver.   Not used.
                        0,                             // Length of buffer in bytes.
                        &ReturnedLength,               // Bytes placed in outbuf.  Should be 0.
                        NULL                           // NULL means wait till I/O completes.
                        ))                            // Did the IOCTL succeed?
        {
//                Ioctl failed
                        return(-1);
        }

        return 0;
   }





static int GetRegState(int reg, ULONG *retVal) //Given a register, return the current contents of the register
   {
        XGPIOPORT_WRITE_INPUT tbuf;
        ULONG                 ReturnedLength; // Number of bytes returned in output buffer
        ULONG                 Iocontrol;

        switch (reg)
        {
            case GPIO_REG:
                Iocontrol = IOCTL_P60xBO_XGPIO_GET_LEVEL;
                break;
            case DIR_REG:
                Iocontrol = IOCTL_P60xBO_XGPIO_GET_DIR;
                break;
            default:
                return(-1);
        }

        //        if (ioctl(fd_p60xbo_xgp, P60xBO_XGPIO_GET_LEVEL, &tbuf))
        if (!DeviceIoControl(
                        fd_p60xbo_xgp,                 // Handle to device
                        Iocontrol,                     // IO Control code for Write
                        NULL,                          // Buffer to driver.  Holds port & data.
                        0,                             // Length of buffer in bytes.
                        &tbuf,                         // Buffer from driver.
                        sizeof(XGPIOPORT_WRITE_INPUT), // Length of buffer in bytes.
                        &ReturnedLength,               // Bytes placed in outbuf.  Should be 0.
                        NULL                           // NULL means wait till I/O completes.
                        ))                            // Did the IOCTL succeed?
        {
        //        Ioctl failed
            printf("GetRegState failed:\n");
      		if (reg == GPIO_REG)
			      printf("   LEV");
		      else
			      printf("   DIR");
		      printf(" =0x%02x 0x%02x  retLen =%d\n", tbuf.CharData[0], tbuf.CharData[1], ReturnedLength);
            return(-1);
        }

       *retVal = (ULONG)tbuf.CharData[0]; 
        return 0;
   }


static int GetPinState(int pin_number, ULONG *retVal) //Given a GPIO pin, return the current state of that pin (High/Low)
   {
        XGPIOPORT_WRITE_INPUT tbuf;
        ULONG                 ReturnedLength; // Number of bytes returned in output buffer

        if ( (pin_number <0) || (pin_number >7)) {
            printf("ERROR:  wrong pin number %d (0..7)\n", pin_number);
            return(-1);
        }

        //        if (ioctl(fd_p60xbo_xgp, P60xBO_XGPIO_GET_LEVEL, &tbuf))
        if (!DeviceIoControl(
                        fd_p60xbo_xgp,                 // Handle to device
                        IOCTL_P60xBO_XGPIO_GET_LEVEL,  // IO Control code for Write
                        NULL,                          // Buffer to driver.  Holds port & data.
                        0,                             // Length of buffer in bytes.
                        &tbuf,                         // Buffer from driver.
                        sizeof(XGPIOPORT_WRITE_INPUT), // Length of buffer in bytes.
                        &ReturnedLength,               // Bytes placed in outbuf.  Should be 0.
                        NULL                           // NULL means wait till I/O completes.
                        ))                             // Did the IOCTL succeed?
        {
        //        Ioctl failed
            printf("GetPinState failed:\n");
            printf("   LEV =0x%02x 0x%02x  retLen =%d\n", tbuf.CharData[0], tbuf.CharData[1], ReturnedLength);
            return(-1);
        }

       *retVal = (ULONG)((tbuf.CharData[0] >> pin_number) & 1); 
        return 0;
   }


static int GetPinDirection(int pin_number, ULONG *retVal) //Given a GPIO pin, return the current direction of that pin (Input/Output)
   {
        XGPIOPORT_WRITE_INPUT tbuf;
        ULONG                 ReturnedLength; // Number of bytes returned in output buffer

        if ( (pin_number <0) || (pin_number >7)) {
             printf("ERROR:  wrong pin number %d (0..7)\n", pin_number);
           return(-1);
        }

        //        if (ioctl(fd_p60xbo_xgp, P60xBO_XGPIO_GET_DIR, &tbuf))
        if (!DeviceIoControl(
                        fd_p60xbo_xgp,                 // Handle to device
                        IOCTL_P60xBO_XGPIO_GET_DIR,    // IO Control code for Write
                        NULL,                          // Buffer to driver.  Holds port & data.
                        0,                             // Length of buffer in bytes.
                        &tbuf,                         // Buffer from driver.
                        sizeof(XGPIOPORT_WRITE_INPUT), // Length of buffer in bytes.
//                      0,
                        &ReturnedLength,               // Bytes placed in outbuf.  Should be 0.
                        NULL                           // NULL means wait till I/O completes.
                        ))                             // Did the IOCTL succeed?
        {
        //        Ioctl failed
            printf("GetPinDirection failed:\n");
            printf("   DIR =0x%02x 0x%02x  retLen =%d\n", tbuf.CharData[0], tbuf.CharData[1], ReturnedLength);
            return(-1);
        }


       *retVal = (ULONG)((tbuf.CharData[0] >> pin_number) & 1); 
        return 0;
   }

// Reset XGpio to all output, all levels zero.
static void ResetXGpioInterface(void)
   {
        XGPIOPORT_WRITE_INPUT tbuf;
        ULONG                 ReturnedLength; // Number of bytes returned in output buffer
        BOOL                  IoctlResult;

        tbuf.CharData[0] = 0;        // all output
        tbuf.CharData[1] = 0xff;         // all pins affected

//        ioctl(fd_p60xbo_xgp, P60xBO_XGPIO_SET_DIR, &tbuf);
//        if (ioctl(fd_p60xbo_xgp, P60xBO_XGPIO_SET_LEVEL, &tbuf))
        IoctlResult = DeviceIoControl(
                        fd_p60xbo_xgp,                 // Handle to device
                        IOCTL_P60xBO_XGPIO_SET_LEVEL,  // IO Control code for Write
                        &tbuf,                         // Buffer to driver.  Holds port & data.
                        sizeof(XGPIOPORT_WRITE_INPUT), // Length of buffer in bytes.
                        NULL,                          // Buffer from driver.   Not used.
                        0,                             // Length of buffer in bytes.
                        &ReturnedLength,               // Bytes placed in outbuf.  Should be 0.
                        NULL                           // NULL means wait till I/O completes.
                        );


        tbuf.CharData[0] = 0;        // all outputs zero
        tbuf.CharData[1] = 0xff;     // all pins affected

//        ioctl(fd_p60xbo_xgp, P60xBO_XGPIO_SET_LEVEL, &tbuf);
        IoctlResult = DeviceIoControl(
                        fd_p60xbo_xgp,                 // Handle to device
                        IOCTL_P60xBO_XGPIO_SET_DIR,    // IO Control code for Write
                        &tbuf,                         // Buffer to driver.  Holds port & data.
                        sizeof(XGPIOPORT_WRITE_INPUT), // Length of buffer in bytes.
                        NULL,                          // Buffer from driver.   Not used.
                        0,                             // Length of buffer in bytes.
                        &ReturnedLength,               // Bytes placed in outbuf.  Should be 0.
                        NULL                           // NULL means wait till I/O completes.
                        );



        return ;
   }

static void ExitXGpioInterface(void)
    {
        // nothing surprising
    CloseHandle(fd_p60xbo_xgp);

    }


#endif  /* __HR_XGPIO_IF_H */
